/**
 * \file formula.cpp
 * \author Léo Perrin
 * \date 2011/11/22
 */

#include <vector>
#include <iostream>
#include <fstream>

#include "formula.h"

// This class has the exit codes 3xx

Formula::Formula(VariableSet * S)
{
	varSet = S;
	newClause();
}


void Formula::newClause()
{
	bufferClause = new Clause();
}


void Formula::addLit(unsigned int s, bool sign, unsigned int i, unsigned int j)
{
	if (sign)
		bufferClause->addLit((int)varSet->getVarCode(s,i,j));
	else
		bufferClause->addLit((-1)*(int)varSet->getVarCode(s,i,j));
}


void Formula::addLit(unsigned int s, bool sign, unsigned int i, unsigned int j, unsigned int k)
{
	if (sign)
		bufferClause->addLit((int)varSet->getVarCode(s,i,j,k));
	else
		bufferClause->addLit((-1)*(int)varSet->getVarCode(s,i,j,k));
}


void Formula::pushClause()
{
	cnfs.push_back(bufferClause);
//        std::cout<<std::endl<<cnfs.size()<<" clauses in the formula";
}


void Formula::restriction(std::vector<int> assign)
{
	for(unsigned int i=0; i<cnfs.size(); i++)
		if (cnfs[i]->restriction(assign))
		{
			cnfs.erase(cnfs.begin()+i);
		}
}


void Formula::toFile(std::ostream * output, std::string comment)
{
	(*output) << "c " << comment << "\n"
		<< "p cnf " << varSet->getCard()
		<< " "<< cnfs.size() << "\n";
	for (unsigned int i=0; i<cnfs.size(); i++)
		(*output) << cnfs[i]->toString() + "\n";
}
